const router = require('express').Router();

router.get('/cvr/:cvr', (req, res, next) => {
    const cvr_regex = /^\d{8}$/g;
    const cvr_api = cvr => `https://cvrapi.dk/api?search=${cvr}&country=dk`;

    if (!cvr_regex.test(req.params.cvr)) {
        res.statusCode = 400;
        return next({name: 'Server-side validering', message: 'Forkert CVR format.'});
    }

    fetch(cvr_api(req.params.cvr), {
        headers: {
            'User-Agent': 'frifaktura.dk | n@nikobojs.com'
        }
    }).then(async (cvrres) => {

        let json_body = null;
        if (cvrres.headers.get('Content-Type').includes('application/json')) {
            json_body = await cvrres.json();
        } else {
            throw new Error('cvrapi.dk did not return JSON!');
        }

        if(cvrres.status == 404) {
            res.statusCode = 404;
            return next({name: 'kunne ikke finde firma'});
        }
        else if (cvrres.status != 200){
            res.statusCode = 500;
            return next({name: 'cvr api returnerede en fejl', message: ''});
        }

        res.json(json_body);
    }).catch(err => {
        console.error(err)
        res.statusCode = 500;
        return next({name: 'cvrapi.dk integrationen virker ikke p.t :/', message: ''});
    });
});


module.exports = router;
