const router = require('express').Router();
const config = require('../config');

function onlyLetters(str) {
    return str
        .split('')
        .filter(
            l => 'abcdefghijklmnopqrstuvwxyz-'
                .split('')
                .includes(l)
        )
        .join('');
}

router.use('/pdf', require('./pdfgen'));
router.use('/pdfgen/:template', (req, res, next) => {
    res.redirect(301, '/pdf/' + onlyLetters(req.params.template));
});

router.use('/api', require('./api'));

router.get('/', (req, res) => {
    res.redirect(`/pdfgen/${config.default_template}`);
});

router.use((error, req, res, next) => {
    console.error(error);
    if(!res.headersSent){
        if(res.statusCode === 200) res.statusCode = 500;
        res.json({name: error.name, message: error.message});
    }
});

router.use((req, res) => {
    res.status(404).json({name: 'not found', message: ''});
});

module.exports = router;
