window.pdfgen = {
    data: {},
    conf: {
        disabled: []
    }
};

const $ = str => {
    const elems = document.querySelectorAll(str);
    if(!elems) throw new Error(`DOM elem '${str}' not found`);
    return (elems.length == 1) ? elems[0] : elems;
};

function merge_obj(obj1, obj2){
    if(!obj1) return obj2;
    Object.keys(obj2).forEach(k => {
        if(typeof(obj2[k]) !== 'object'){
            obj1[k] = obj2[k];
        }
        else{
            obj1[k] = merge_obj(obj1[k], obj2[k]);
        }
    });
    return obj1;
}


function error(err){
    console.log('error(): ', err);
    if(err.name && err.message){
        alert(`${err.name}\n${err.message}`);
    }
    else if(typeof(err) === 'string'){
        alert(err);
    }
    else{
        console.error(err);
    }

    if(err.el && typeof(err.el) == 'object' && document.body.contains(err.el)){
        setTimeout(() => {
            err.el.focus();
            err.el.select();
        }, 100);
    }

    return undefined;
}

async function cvrapi(el){
    const cvr = el.value;
    let resjson = null;
    if(!/\d{8}/g.test(cvr)) return error({
        el: el,
        name: 'Forkert CVR format',
        message: ' '
    });
    const resobj = await fetch(`/api/cvr/${cvr}`);
    try{ resjson = await resobj.json(); }
    catch(e) {
        console.log('json error:', e);
        return error({
            el: el,
            name: 'Kunne ikke parse respons body til JSON',
            message: 'Serveren svarede ikke korrekt, skriv gerne en fejlrapport.'
        });
    }

    if(resobj.status === 404) {
        return error({
            el: el,
            name: 'Kunne ikke finde et firma',
            message: `med CVR-nummer ${cvr}`
        });
    }

    if(!resobj.ok) return error({
        el: el,
        name: 'Server error',
        message: el.error
    });

    return resjson;
}


function format_date(d = new Date()) {
    const is_date = Object.prototype.toString.call(d) === '[object Date]';
    if (!is_date && typeof d === 'string') {
        try {
            d = new Date(d);
        } catch (e) {
            console.error('Invalid date!');
            throw e;
        }
    } else if(!is_date) {
        throw new Error('Invalid date!');
    }

    return `${d.getDate()}/${d.getMonth()+1}/${d.getFullYear()}`;
}

function populate(){
    return new Promise(async (resolve, reject) => {
        let elems = Array.from(document.getElementsByClassName('var'));
        elems = elems.filter(el => {
            return !window.pdfgen.conf.disabled.includes(el.id)
        });

        const addValue = (key, val) => {
            if(key.includes('.')){
                const all_keys = [].concat(key.split('.'))
                const keys = key.split('.').reverse();
                let obj = {};
                obj[keys.shift()] = val;

                keys.forEach((k, i) => {
                    const deep = {};
                    deep[k] = obj;
                    obj = deep;
                });

                window.pdfgen.data = merge_obj(window.pdfgen.data, obj);
            }
            else{
                window.pdfgen.data[key] = val;
            }
        }

        await Promise.all(elems.map(async (el) => {

            console.log({ dataset: el.dataset })
            if (Object.keys(el.dataset).includes('optional') && ['', undefined, null].includes(el.value)) {
                return;
            }

            const humanName = el.placeholder || el.id || el.dataset?.var || 'ukendt';

            if(el.dataset.is === 'number'){
                try {
                    const number = parseInt(el.value);
                    addValue(el.dataset.var, number);
                }
                catch(e) { throw {
                    el: el,
                    name: `Valideringsfejl for feltet '${humanName}'`,
                    message: 'Værdien kunne ikke konverteres til et tal'
                }; }
            }
            else if(el.dataset.is === 'boolean') {
                if(typeof(el.checked) !== 'boolean') throw {
                    el: el,
                    name: `Valideringsfejl for feltet '${humanName}'`,
                    message: 'Checkboxens værdi er ikke gyldig :S'
                };
                addValue(el.dataset.var, el.checked);
            }
            else if(el.dataset.is === 'string'){
                if(typeof(el.value) !== 'string' || el.value === '') throw {
                    el: el,
                    name: `Valideringsfejl for feltet '${humanName}'`,
                    message: 'Teksten er tom eller ugyldig'
                };
                addValue(el.dataset.var, el.value);
            }
        })).then(() => resolve()).catch(e => {
            if(e.el) {
                e.el.className += ' vali_err';
                setTimeout(() => e.el.className = e.el.className.replace(/\ vali_err/g, ''), 3800);
            }
            return reject(e);
        });
    });
}

// TODO: fix this !
function modal_toggle(name){
    const modal = document.querySelector(`#modal-${name}`);
    if(modal.className.includes('show')){
        modal.className = modal.className.replace(/\ show/g, '');
        // TODO: add window-dom onclick
    }
    else{
        modal.className += ' show';
    }
}


// upload on file input change event
window.addEventListener('DOMContentLoaded', e => {
    const inputs = Array.from(document.querySelectorAll('input.var[type="file"]'));

    // only image fileupload (TODO: why?)
    inputs.filter(el => el.dataset.is === 'image').forEach(el => {
        el.addEventListener('change', async (evt) => {
            if(!window.pdfgen.data.files) window.pdfgen.data.files = {};

            let resjson;
            const data = new FormData();

            if(!el.files[0]) throw {
                el: el,
                name: 'Client-side input validering',
                message: 'Du mangler at vælge en fil til upload'
            };

            data.append('file', el.files[0]);

            const resobj = await fetch('/pdf/upload/image', {
                method: 'POST',
                body: data
            });

            try{ resjson = await resobj.json(); }
            catch(e) {
                throw {
                    el: el,
                    name: 'Kunne ikke parse respons body til JSON',
                    message: 'Serveren svarede ikke korrekt, skriv gerne en fejlrapport.'
                };
            }

            if(!resobj.ok) {
                // TODO: throw or reject? or error()?
                if(!resjson.error) return reject(resjson);
                throw {
                    el: el,
                    name: 'Kunne ikke uploade fil',
                    message: resjson.error
                };
            }

            if(window.file_reset_timer){
                setTimeout(() => {
                    delete window.pdfgen.data.files[el.dataset.var];
                    if(el.dataset.preview){
                        $(`#${el.dataset.preview}`).innerHTML = '';
                    }
                }, window.file_reset_timer);
            }

            if(el.dataset.preview){
                const preview_container = $(`#${el.dataset.preview}`);
                preview_container.innerHTML = '';
                const preview_img = document.createElement('img');
                preview_img.className = 'image_preview';
                preview_img.src = `file/${resjson.fileid}`;
                preview_container.appendChild(preview_img);
            }

            window.pdfgen.data.files[el.dataset.var] = resjson.fileid;
        });
    });

});
