function rain(container, svg_path, opts = {}) {

    // default options
    const o = {
        density:                1000,             // higher means faster spawn interval
        min_duration:           3000,           // minimum animation duration
        max_duration:           5000,           // minimum animation duration
        easing:                 'ease-in',      // use other easings, e.g. 'cubic-bezier(.57,.01,.87,.77)'
        spawnsvg_uncertainty:   500,            // raise to make spawn intervals seem more random
        min_rotation:           200,            // minimum rotation (deg) during full animation cycle
        max_rotation:           1000,           // maximum rotation (deg) during full animation cycle
        min_size:               30,             // minimum svg size in pixels
        max_size:               60,             // maximum svg size in pixels
        start_color:            '#94c3ff5e',    // start animation color, follows same easing
        end_color:              '#ffc3fff0',    // end animation color
        full_screen:            true            // override container size and use full documentElement boundaries
    };

    // apply options arg
    Object.assign(o, opts);

    // set width and height
    let w, h;
    if (o.full_screen) {
        w = o.max_size + Math.max(document.documentElement.clientWidth-o.max_size, window.innerWidth-o.max_size || 0);
        h = Math.max(document.body.scrollHeight, document.body.offsetHeight, document.documentElement.clientHeight, document.documentElement.scrollHeight, document.documentElement.offsetHeight);
    }
    else {
        w = container.offsetWidth;
        h = container.offsetHeight;
    }

    // create subcontainer
    const subcontainer = document.createElement('div');

    // add subcontainer
    let css_string = `
        position: absolute;
        top: -${o.max_size}px;
        height: ${o.full_screen ? h + o.max_size + 'px' : 'calc(100% + ' + o.max_size + 'px)'};
        width: 100%;
        left: 0px;
        overflow: hidden;
        z-index: 1;
    `;
    subcontainer.setAttribute('style', css_string);
    if(!o.full_screen) container.style.position = 'relative'; // force this to make non-full screen overflow:hidden work
    container.appendChild(subcontainer);

    // define create svg function
    const create_svg = () => {
        const svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
        const x = 100+Math.floor(Math.random() * w-200);
        const size = o.min_size + Math.floor(Math.random() * (o.max_size-o.min_size))
        const duration = o.min_duration + Math.floor(Math.random() * (o.max_duration-o.min_duration))
        const duration_css = (duration / 1000).toFixed(1) + 's';
        const spawn_random = Math.floor(Math.random() * o.spawnsvg_uncertainty);
        const rotation = o.min_rotation + Math.floor(Math.random() * (o.max_rotation-o.min_rotation))
        // console.log(`new svg; duration_css is ${duration}; x is ${x}; spawntime is ${spawn_random}, rotation is ${rotation}degs`);

        css_string = `
            position: absolute;
            top: 0px;
            z-index: 1;
            mask-repeat: no-repeat;
            -webkit-mask-repeat: no-repeat;
            mask-size: 100% 100%;
            -webkit-mask-size: 100% 100%;
            width: ${size}px;
            left: ${x}px;
            height: ${size}px;
            background-color: ${o.start_color};
            -webkit-mask-image: url(${svg_path});
            mask-image: url(${svg_path});
            transition: all ${duration_css} ${o.easing};
            -webkit-transition: all ${duration_css} ${o.easing};
        `;

        svg.setAttribute("width", "100");
        svg.setAttribute("height", "100");
        svg.setAttribute("fill", "red");
        svg.setAttribute('style', css_string);

        setTimeout(() => svg.parentNode.removeChild(svg), spawn_random + duration + 500);

        return {svg, spawn_random, rotation};
    };


    setInterval(() => {
        const svg = create_svg();
        subcontainer.appendChild(svg.svg);
        const minus_or_not = !Math.round(Math.random()) ? '-' : '';
        setTimeout(() => {
            svg.svg.style.transform = `translate(0px, ${h + 120}px) rotate(${minus_or_not}${svg.rotation}deg)`;
            svg.svg.style.backgroundColor = o.end_color;
        }, 100 + svg.spawn_random);
    }, 10000/o.density);
}
